#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "gpstdio.h"
#include "gpfont.h"

#include "GpMain.h"
#include "Hiscore.h"


extern GPDRAWSURFACE gpDraw[2];
extern int nflip;
extern HISCORES scores[3];

extern GP_PALETTEENTRY hiscores_Pal[256];
extern const unsigned char hiscores[];


void InputName(char name[12])
{
	const char alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 ";

	int keydata;
	int i=0, j=0;

	GP_HPALETTE h_pal, old_pal;

	// Create palette
	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)hiscores_Pal);

	// Change palette
	GpPaletteSelect(h_pal,NULL);
	GpPaletteRealize();
		
	// Restore palette
	GpPaletteSelect(old_pal,h_pal);
				
	// Release resource
	GpPaletteDelete(h_pal);
	h_pal = NULL;

	gp_str_func.memset((void*)name, 0, 12); // Clear name

	while (1) // name is 12 chars max
	{
		GpBitBlt(NULL, 
		         &gpDraw[nflip], 
				0, 0, 
				320, 240, 
				(unsigned char*)hiscores, 
				0, 0, 
				320, 240);

		GpTextOut(NULL, &gpDraw[nflip], 70, 120, (char*)" THIS IS AN HISCORE !!!", 0xff);
		GpTextOut(NULL, &gpDraw[nflip], 70, 150, (char*)"Please enter your name:", 0xff);

		GpKeyGetEx(&keydata);
		
		if ((keydata & GPC_VK_START) && GpKeyChanged())
		{
			break;
		}
		else if (keydata & GPC_VK_UP)
		{
			i--;
			if (i<0)
				i=sizeof(alphabet)-2;

			GpThreadSleep(80);
		}
		else if (keydata & GPC_VK_DOWN)
		{
			i++;
			if (i>sizeof(alphabet)-2)
				i=0;

			GpThreadSleep(80);
		}
		else if ((keydata & GPC_VK_FA) && GpKeyChanged())
		{
			j++;
			if (j>12)
				j--;
		}
		else if ((keydata & GPC_VK_FB) && GpKeyChanged())
		{
			name[j] = alphabet[sizeof(alphabet)-2];
			j--;
			if (j<0)
				j++;
			name[j] = alphabet[sizeof(alphabet)-2];
		}

		name[j] = alphabet[i];
		GpTextOut(NULL, &gpDraw[nflip], 70, 180, (char*)name, 0xff);

		GpSurfaceFlip(&gpDraw[nflip++]);
		nflip %= 2;
	}

	return;
}


void DisplayHiscores(void)
{
	int i;
	char tmp_score[16];
	int keydata;

	GP_HPALETTE h_pal, old_pal;

	// Create palette
	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)hiscores_Pal);

	// Change palette
	GpPaletteSelect(h_pal,NULL);
	GpPaletteRealize();
		
	// Restore palette
	GpPaletteSelect(old_pal,h_pal);
				
	// Release resource
	GpPaletteDelete(h_pal);
	h_pal = NULL;

	GpBitBlt(NULL, 
	         &gpDraw[nflip], 
			0, 0, 
			320, 240, 
			(unsigned char*)hiscores, 
			0, 0, 
			320, 240);

	GpTextOut(NULL, &gpDraw[nflip], 60, 140, (char*)"1st", 0x01);
	GpTextOut(NULL, &gpDraw[nflip], 60, 160, (char*)"2nd", 0x01);
	GpTextOut(NULL, &gpDraw[nflip], 60, 180, (char*)"3rd", 0x01);

	for (i=0; i<3; i++)
	{
		gp_str_func.sprintf((char*)tmp_score, "%d", scores[i].score);
		GpTextOut(NULL, &gpDraw[nflip], 100, 140+i*20, (char*)scores[i].name, 0x01);
		GpTextOut(NULL, &gpDraw[nflip], 220, 140+i*20, (char*)tmp_score, 0x01);
	}

	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	while (1)
	{
		GpKeyGetEx(&keydata);
		
		if (((keydata & GPC_VK_FA) || (keydata & GPC_VK_FB) || (keydata & GPC_VK_START) || (keydata & GPC_VK_SELECT)) && GpKeyChanged())
		{
			break;
		}
	}

	// Fill screen with black because we are going to change palette
	GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	SelectMode();
}


void LoadScores(void)
{
	unsigned long size;
	
	ERR_CODE err_code;
	F_HANDLE h_rfile;
	unsigned long n_dummy;

	err_code = GpRelativePathSet("gp:\\gpmm\\gptetris");
	if (err_code != SM_OK)
	{
		return;
	}

	// Get the file size
	err_code = GpFileGetSize("gptetris.dat", &size);
	if (err_code != SM_OK)
	{
		return;
	}

	// If size = 0, file is null
	if (size == 0)
	{
		return;
	}

	err_code = GpFileOpen("gptetris.dat", OPEN_R, &h_rfile);
	if (err_code != SM_OK)
	{
		return;
	}
	
	err_code = GpFileRead(h_rfile, scores, size, (ulong*)&n_dummy);
	if (err_code != SM_OK)
	{
		return;
	}
	GpFileClose(h_rfile);
} 


void WriteScores(void)
{
	ERR_CODE err_code;
	F_HANDLE h_file;

	GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
	GpTextOut(NULL, &gpDraw[nflip], 20, 100, (char*)"Please wait while writing Hiscores", 0xff);
	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	// Remember that GpFatInit() must have been called
	// before we do any operation on the SMC
	err_code = GpRelativePathSet("gp:\\gpmm\\gptetris");
	if (err_code != SM_OK)
	{
		GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
		GpTextOut(NULL, &gpDraw[nflip], 20, 100, (char*)"Error directory GP:\\GPMM\\GPTETRIS not found", 0xff);
		GpSurfaceFlip(&gpDraw[nflip++]);
		nflip %= 2;

		GpThreadSleep(1000);

		return;
	}

	err_code = GpFileCreate("gptetris.dat", ALWAYS_CREATE, &h_file);
	if (err_code != SM_OK)
	{
		GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
		GpTextOut(NULL, &gpDraw[nflip], 20, 100, (char*)"Error creating save file", 0xff);
		GpSurfaceFlip(&gpDraw[nflip++]);
		nflip %= 2;

		GpThreadSleep(1000);

		return;
	}

	err_code = GpFileWrite(h_file, scores, 48);
	if (err_code != SM_OK)
	{
		GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
		GpTextOut(NULL, &gpDraw[nflip], 20, 100, (char*)"Error writing save file", 0xff);
		GpSurfaceFlip(&gpDraw[nflip++]);
		nflip %= 2;

		GpThreadSleep(1000);

		return;
	}


	GpFileClose(h_file);
}
