#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "gpfont.h"
#include "gpmm.h"
#include "gpstdio.h"
#include "gpstream.h"

#include "GpMain.h"
#include "MathFunc.h"
#include "GraphicFunc.h"
#include "TetrisIntro.h"
#include "Hiscore.h"

#define TRUE	1
#define FALSE	0

extern GP_PALETTEENTRY mainmenu_Pal[];
extern GP_PALETTEENTRY chooselevel_Pal[];
extern GP_PALETTEENTRY selectmode_Pal[];
extern GP_PALETTEENTRY level1_Pal[];
extern GP_PALETTEENTRY level2_Pal[];
extern GP_PALETTEENTRY level3_Pal[];
extern const unsigned char mainmenu[];
extern const unsigned char press_start[];
extern const unsigned char selectmode[];
extern const unsigned char chooselevel[];
extern const unsigned char level1[];
extern const unsigned char level2[];
extern const unsigned char level3[];
unsigned char tetris[64*32];

extern const unsigned short delete[29738];
extern const unsigned short putblock[18465];
extern const unsigned short rotate[5361];

extern TGpStreamBuf g_stream_buf;

int start_blocks;					// Number of initial rows on the map
int block[4][4];					// Current block
int nextblock[4][4];				// Current block
int block_x, block_y;				// Block location
int current_block;				// Current block type
int blocksprite;					// Current block sprite y co-ordinate in the level picture
int nextblocksprite;				// Next block sprite y co-ordinate in the level picture
int next_block;					// Next block
int gamefield[10][20];				// The game field (width 10; height 20)
int x_size, y_size;				// Size of the block
int start_x = 110, start_y = 30;	// Game field (starting point)
int stop_x = 210, stop_y = 230;		// Game field (ending point)
int speed;						// Speed of the blocks
int level;						// Current level
int score, dels;					// Game score and number of rows deleted
int drop=0;						// are we dropping the current block?
int timer = 0;					// Timer used to know when the block should go down
unsigned char shownext=1;			// do you want to show next block?

HISCORES scores[3];

unsigned char* current_level = (unsigned char*)level1;
GP_PALETTEENTRY* current_Pal = (GP_PALETTEENTRY*)level1_Pal;

GPDRAWSURFACE gpDraw[2];
int nflip;


void ClearBlock(int block[4][4])
{
	int i, j;
 	for (i = 0; i < 4; i++)
	{
		for (j = 0; j < 4; j++)
		{


			block[i][j] = 0;
		}
	}
}

void GenerateBlock(int kind, int block[4][4])
{
 	ClearBlock(block);

	switch (kind)
	{
		//	  [][]
		//	[][]
		case  1: block[1][0] = block[2][0] = block[0][1] = block[1][1] = 1; blocksprite = 160; break;
		case 11: block[0][0] = block[0][1] = block[1][1] = block[1][2] = 1; blocksprite = 160; break;
		case 21: block[1][0] = block[2][0] = block[0][1] = block[1][1] = 1; blocksprite = 160; break;
		case 31: block[0][0] = block[0][1] = block[1][1] = block[1][2] = 1; blocksprite = 160; break;

		//	[][]
		//	  [][]
		case  2: block[0][0] = block[1][0] = block[1][1] = block[2][1] = 1; blocksprite = 192; break;
		case 12:	block[1][0] = block[1][1] = block[0][1] = block[0][2] = 1; blocksprite = 192; break;
		case 22:	block[0][0] = block[1][0] = block[1][1] = block[2][1] = 1; blocksprite = 192; break;
		case 32:	block[1][0] = block[1][1] = block[0][1] = block[0][2] = 1; blocksprite = 192; break;

		//	[][]
		//	[][]
		case  3: block[0][0] = block[0][1] = block[1][0] = block[1][1] = 1; blocksprite = 176; break;
		case 13: block[0][0] = block[0][1] = block[1][0] = block[1][1] = 1; blocksprite = 176; break;
		case 23: block[0][0] = block[0][1] = block[1][0] = block[1][1] = 1; blocksprite = 176; break;
		case 33: block[0][0] = block[0][1] = block[1][0] = block[1][1] = 1; blocksprite = 176; break;

		//	  []
		//	[][][]
		case  4: block[1][0] = block[0][1] = block[1][1] = block[2][1] = 1; blocksprite = 176; break;
		case 14: block[0][0] = block[0][1] = block[0][2] = block[1][1] = 1; blocksprite = 176; break;
		case 24: block[0][0] = block[1][0] = block[2][0] = block[1][1] = 1; blocksprite = 176; break;
		case 34: block[0][1] = block[1][0] = block[1][1] = block[1][2] = 1; blocksprite = 176; break;

		//	    []
		//	[][][]
		case  5: block[2][0] = block[0][1] = block[1][1] = block[2][1] = 1; blocksprite = 192; break;
		case 15: block[0][0] = block[0][1] = block[0][2] = block[1][2] = 1; blocksprite = 192; break;
		case 25: block[0][0] = block[1][0] = block[2][0] = block[0][1] = 1; blocksprite = 192; break;
		case 35: block[0][0] = block[1][0] = block[1][1] = block[1][2] = 1; blocksprite = 192; break;

		//	[]
		//	[][][]
		case  6: block[0][0] = block[0][1] = block[1][1] = block[2][1] = 1; blocksprite = 208; break;
		case 16: block[0][0] = block[1][0] = block[0][1] = block[0][2] = 1; blocksprite = 208; break;
		case 26: block[0][0] = block[1][0] = block[2][0] = block[2][1] = 1; blocksprite = 208; break;
		case 36: block[1][0] = block[1][1] = block[1][2] = block[0][2] = 1; blocksprite = 208; break;

		//	[][][][]
		case  7: block[0][0] = block[1][0] = block[2][0] = block[3][0] = 1; blocksprite = 144; break;
		case 17: block[1][0] = block[1][1] = block[1][2] = block[1][3] = 1; blocksprite = 144; break;
		case 27: block[0][0] = block[1][0] = block[2][0] = block[3][0] = 1; blocksprite = 144; break;
		case 37: block[1][0] = block[1][1] = block[1][2] = block[1][3] = 1; blocksprite = 144; break;

		default: break;
	}

	if ((kind == 7) || (kind == 27))
	{
		x_size = 4;
		y_size = 1;
	}
	else if ((kind == 17) || (kind == 37))
	{
		x_size = 1;
		y_size = 4;
	}
	else if (kind % 10 == 3)
	{
		x_size = 2;
		y_size = 2;
	}
	else if ((kind / 10 == 0) || (kind / 10 == 2))
	{
		x_size = 3;
		y_size = 2;
	}
	else if ((kind / 10 == 1) || (kind / 10 == 3))
	{
		x_size = 2;
		y_size = 3;
	}

	current_block = kind;
}


void GenerateNew(void)
{
	int current_block = next_block;
	next_block = Random(7)+1;

	GenerateBlock(next_block, nextblock);
	nextblocksprite = blocksprite;
	
	GenerateBlock(current_block, block);

	block_x = 3;
	block_y = 0;

	if (!CanPut())
		EndGame();
}


void PutBlock(void)
{
	int i, j;

	for (i=0; i<4; i++)
	{
		for (j=0; j<4; j++)
		{
			if (block[i][j] == 1)
				gamefield[block_x+i][block_y+j] = block[i][j];
		}
	}

	GpPcmPlay((unsigned short*)putblock,sizeof(putblock), 0);
	GpThreadSleep(50/2); // 50 milliseconds delay

	DeleteRow();
}


void RotateBlockRight(void)
{
	int old_block = current_block;
	current_block += 10;
	if (current_block > 40) current_block -= 40;
	GenerateBlock(current_block, block);
	if (!CanPut())
	{
		current_block = old_block;
		GenerateBlock(current_block, block);
	}

	GpPcmPlay((unsigned short*)rotate,sizeof(rotate), 0);
	GpThreadSleep(50/2); // 50 milliseconds delay
}


void RotateBlockLeft(void)
{
	int old_block = current_block;
	current_block -= 10;
	if (current_block < 0) current_block += 40;
	GenerateBlock(current_block, block);
	if (!CanPut())
	{
		current_block = old_block;
		GenerateBlock(current_block, block);
	}

	GpPcmPlay((unsigned short*)rotate,sizeof(rotate), 0);
	GpThreadSleep(50/2); // 50 milliseconds delay
}


int CanPut(void)
{
	int i, j;
	for (i=0; i<4; i++)
	{
		for (j=0; j<4; j++)
		{
			if (block[i][j] == 1)
			{
				if (block_x + i > 9) block_x -= (block_x+i-9); // where are near a border, move
				if (block_x + i < 0) block_x -= (block_x+i); // where are near a border, move
				if (block_y + j > 19) return FALSE;
				if (gamefield[block_x+i][block_y+j] == 1) return FALSE;
			}
		}
	} 
	return TRUE;
}



void DrawMap(void)
{
	char string[256];
	int i, j;

	GpBitBlt(NULL, 
	         &gpDraw[nflip], 
	         0, 0, 
	         320, 240, 
	         (unsigned char*)current_level,
	         0, 0, 
	         336, 240);

	if (shownext==1)
	{
		GpRectFill(NULL, &gpDraw[nflip], 219, 174, 92, 57, 0x00);
		GpRectFill(NULL, &gpDraw[nflip], 220, 175, 90, 55, 0x01);
	}

	for (i=0; i<10; i++)
	{
 		for (j=0; j<20; j++)
		{
			// Draw blocks already on the gamefield
			if (gamefield[i][j] == 1)
				GpTransBlt(NULL, 
			         &gpDraw[nflip], 
					start_x+i*10, start_y+j*10, 
					16, 16, 
					(unsigned char*)current_level,
					320, 224, 
					336, 240,
					0xff);
		}
	}

	for (i=0; i<4; i++)
	{
		for (j=0; j<4; j++)
		{
			// Draw current block
			if (block[i][j] == 1)
				GpTransBlt(NULL, 
			         &gpDraw[nflip], 
					start_x+block_x*10+i*10, start_y+block_y*10+j*10, 
					16, 16, 
					(unsigned char*)current_level,
					320, blocksprite, 
					336, 240,
					0xff);

			// Draw next block
			if (nextblock[i][j] == 1 && shownext == 1)
				GpTransBlt(NULL, 
			         &gpDraw[nflip], 
					250+i*10, 205+j*10, 
					16, 16, 
					(unsigned char*)current_level,
					320, nextblocksprite, 
					336, 240,
					0xff);
		}
	}


	// Display score and other infos
	gp_str_func.sprintf((char*)string, "Score:%d", score);
	GpTextOut(NULL, &gpDraw[nflip], 217, 30, (char*)string, 0x00);

	gp_str_func.sprintf((char*)string, "Lines:%d", dels);
	GpTextOut(NULL, &gpDraw[nflip], 217, 50, (char*)string, 0x00);

	gp_str_func.sprintf((char*)string, "Level:%d", level);
	GpTextOut(NULL, &gpDraw[nflip], 217, 70, (char*)string, 0x00);

	if (shownext==1)
		GpTextOut(NULL, &gpDraw[nflip], 225, 180, (char*)"Next block", 0x00);

	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	GpThreadSleep(30/2); // 30 milliseconds delay
}


void DeleteRow(void)
{
	int sum, i, j, k, n_tick, del = 0;

	// Draw map before deleting blocks so that primary and secondary buffer are identical
	DrawMap();

	for (i=19; i>=0; i--)
	{
		sum = 0;
		for (j=0; j<10; j++)
			sum += gamefield[j][i];
		if (sum == 10)
		{
			for (j=i; j>=1; j--)
				for (k=0; k<10; k++)
					gamefield[k][j] = gamefield[k][j-1];
			i += 2;
			del++;
		}

	}


	sum = 0;

	for (i=0; i<20; i++)
	{
		for (j=0; j<10; j++)
		{
			sum += gamefield[j][i];
		}
	}

	// If the gamefiled was cleared
	if (sum == 0)
	{
		for (i=0; i<start_blocks; i++)
		{
			for (j=0; j<10; j++)
			{
				if (Random(2)) gamefield[j][19-i] = 1;
			}
		}
	}

	if (del > 0)

	{
		GpPcmPlay((unsigned short*)delete,sizeof(delete), 0);
		GpThreadSleep(50/2); // 50 milliseconds delay

		// Player made a Tetris!!
		if (del == 4)
		{
			int x, y;

			// Extract the tetris sprite from the current level picture
			for (x=0; x<64; x++)
			{
				for (y=0; y<32; y++)
				{
					tetris[x*32+y] = current_level[(335+x)*240+y];
				}
			}

			for (i=1; i<6; i++)
			{
				ZoomPicture(&gpDraw[nflip], i, (unsigned char*)tetris, 64, 32, 0xff);

				GpSurfaceFlip(&gpDraw[nflip++]);
				nflip %= 2;

				n_tick = GpTickCountGet();
				while ( ( GpTickCountGet() - n_tick ) < 50);
			}


			n_tick = GpTickCountGet();
			while ((GpTickCountGet() - n_tick) < 1000);

			score += 2500;
		}
		else if (del == 3)
		{
			score += 500;
		}
		else if (del == 2)
		{
			score += 300;
		}
		else if (del == 1)
		{
			score += 100;

		}

		dels += del;
	}

	level = (dels/8)+1;
	if (level > 10)
		level = 10;

	switch(level)
	{
		case  1: speed = 40; break;
		case  2: speed = 30; break;
		case  3: speed = 20; break;
		case  4: speed = 15; break;
		case  5: speed = 10; break;
		case  6: speed =  8; break;
		case  7: speed =  6; break;
		case  8: speed =  4; break;
		case  9: speed =  3; break;
		case 10: speed =  2; break;
	}
}


void NewGame(void)
{
	int i, j;

 	for (i=0; i<20; i++)
	{
		for (j=0; j<10; j++)
		{
			gamefield[j][i] = 0;
		}
	}

	timer = 0;
	score = 0;
	dels = 0;
	speed = 50;
	level = 1;

	next_block = Random(7)+1;
	GenerateNew();
	DrawMap();

	for (i=0; i<start_blocks; i++)
	{
		for (j=0; j<10; j++)
		{
			if (Random(2)) gamefield[j][19-i] = 1;
		}
	}
}


void EndGame(void)
{
	GpTextOut(NULL, &gpDraw[nflip^1], 230, 120, (char*)"GAMEOVER", 0x00);
	GpTextOut(NULL, &gpDraw[nflip^1], 220, 140, (char*)"Press START", 0x00);

	while (!(GpKeyGet() & GPC_VK_START));

	// Wait a few milliseconds so that we don't get two START hit
	GpThreadSleep(200/2); // 50 milliseconds delay

	if (score > scores[0].score) // It's an hiscore
	{
		scores[2].score = scores[1].score; gp_str_func.strcpy(scores[2].name, scores[1].name);
		scores[1].score = scores[0].score; gp_str_func.strcpy(scores[1].name, scores[0].name);
		scores[0].score = score;

		InputName(scores[0].name);

		// Write the scores to a file
		WriteScores();
	}
	else if (score > scores[1].score) // It's an hiscore
	{
		scores[2].score = scores[1].score; gp_str_func.strcpy(scores[2].name, scores[1].name);
		scores[1].score = score;

		InputName(scores[1].name);

		// Write the scores to a file
		WriteScores();
	}
	else if (score > scores[2].score)
	{
		scores[2].score = score;

		InputName(scores[2].name);

		// Write the scores to a file
		WriteScores();
	}

	MainMenu();
}


void MainMenu(void)
{
	GP_HPALETTE h_pal, old_pal;
	int frame=0;

	// Create palette
	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)mainmenu_Pal);

	// Change palette
	GpPaletteSelect(h_pal,NULL);
	GpPaletteRealize();
		
	// Restore palette
	GpPaletteSelect(old_pal,h_pal);
				
	// Release resource
	GpPaletteDelete(h_pal);
	h_pal = NULL;

	while (!(GpKeyGet() & GPC_VK_START))
	{
	GpBitBlt(NULL, 
	         &gpDraw[nflip], 
	         0, 0, 
	         320, 240, 
	         (unsigned char*)mainmenu, 
	         0, 0, 
	         320, 240);

	if (frame < 32)
	{
		GpBitBlt(NULL, 
		         &gpDraw[nflip], 
				100, 120,
				128, 35, 
				(unsigned char*)press_start, 
				0, 0, 
				128, 40);
	}
	if (frame >= 64)
		frame=0;

	frame++;

	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	};

	GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	GpPcmPlay((unsigned short*)delete,sizeof(delete), 0);
	GpThreadSleep(50/2); // 50 milliseconds delay

	SelectMode();
}


void SelectMode(void)
{
	int keydata;
	int cmode = 1;

	GP_HPALETTE h_pal, old_pal;

	// Create palette
	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)selectmode_Pal);

	// Change palette
	GpPaletteSelect(h_pal,NULL);
	GpPaletteRealize();
		
	// Restore palette
	GpPaletteSelect(old_pal,h_pal);
				
	// Release resource
	GpPaletteDelete(h_pal);
	h_pal = NULL;


	while (1)
	{
		GpKeyGetEx(&keydata);
		
		if ((keydata & GPC_VK_DOWN) && GpKeyChanged())
		{
			GpPcmPlay((unsigned short*)rotate,sizeof(rotate), 0);
			GpThreadSleep(50/2); // 50 milliseconds delay

			cmode++;
			if (cmode > 3) cmode = 1;
		}
		else if ((keydata & GPC_VK_UP) && GpKeyChanged())
		{
			GpPcmPlay((unsigned short*)rotate,sizeof(rotate), 0);
			GpThreadSleep(50/2); // 50 milliseconds delay

			cmode--;
			if (cmode < 1) cmode = 3;
		}
		else if (((keydata & GPC_VK_FA) || (keydata & GPC_VK_START) || (keydata & GPC_VK_SELECT)) && GpKeyChanged())
		{
			break;
		}
		else if ((keydata & GPC_VK_FB) && GpKeyChanged())
		{
			// Fill screen with black because we are going to change palette
			GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
			GpSurfaceFlip(&gpDraw[nflip++]);
			nflip %= 2;

			MainMenu();
		}

		GpBitBlt(NULL, 
			&gpDraw[nflip], 
			0, 0,
			320, 240, 
			(unsigned char*)selectmode,
			0, 0, 
			464, 240);

		switch (cmode)
		{
			case  1:
			GpTransBlt(NULL, 
		         &gpDraw[nflip], 
				89, 109, 
				144, 30, 
				(unsigned char*)selectmode,
				320, 210, 
				464, 240,
				0xff);
				break;
			case  2:
			GpTransBlt(NULL, 
		         &gpDraw[nflip], 
				89, 150, 
				144, 30, 
				(unsigned char*)selectmode,
				320, 210, 
				464, 240,
				0xff);
				break;
			case  3:
			GpTransBlt(NULL, 
		         &gpDraw[nflip], 
				89, 191, 
				144, 30, 
				(unsigned char*)selectmode,
				320, 210, 
				464, 240,
				0xff);
				break;
			default:
				break;
		}

		GpThreadSleep(100/2); // 100 milliseconds delay

		GpSurfaceFlip(&gpDraw[nflip++]);
		nflip %= 2;
	}

	GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	GpPcmPlay((unsigned short*)delete,sizeof(delete), 0);
	GpThreadSleep(200/2); // 200 milliseconds delay


	switch (cmode)
	{
		case  1:
			start_blocks=0;
			SelectLevel();
			break;
		case  2:
			start_blocks=5;
			SelectLevel();
			break;
		case  3:
			DisplayHiscores();
			break;
		default:
			break;
	}
}


void SelectLevel(void)
{
	int keydata;
	int clevel = 1;

	GP_HPALETTE h_pal, old_pal;

	// Create palette
	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)chooselevel_Pal);

	// Change palette
	GpPaletteSelect(h_pal,NULL);
	GpPaletteRealize();
		
	// Restore palette
	GpPaletteSelect(old_pal,h_pal);
				
	// Release resource
	GpPaletteDelete(h_pal);
	h_pal = NULL;



	while (1)
	{
		GpKeyGetEx(&keydata);
		
		if ((keydata & GPC_VK_RIGHT) && GpKeyChanged())
		{
			GpPcmPlay((unsigned short*)rotate,sizeof(rotate), 0);
			GpThreadSleep(50/2); // 50 milliseconds delay

			clevel++;
			if (clevel > 3) clevel = 1;
		}
		else if ((keydata & GPC_VK_LEFT) && GpKeyChanged())
		{
			GpPcmPlay((unsigned short*)rotate,sizeof(rotate), 0);
			GpThreadSleep(50/2); // 50 milliseconds delay

			clevel--;
			if (clevel < 1) clevel = 3;
		}
		else if (((keydata & GPC_VK_FA) || (keydata & GPC_VK_START) || (keydata & GPC_VK_SELECT)) && GpKeyChanged())
		{
			break;
		}
		else if ((keydata & GPC_VK_FB) && GpKeyChanged())
		{
			// Fill screen with black because we are going to change palette
			GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
			GpSurfaceFlip(&gpDraw[nflip++]);
			nflip %= 2;

			SelectMode();
		}


		GpBitBlt(NULL, 
			&gpDraw[nflip], 
			0, 0,
			320, 240, 
			(unsigned char*)chooselevel,
			0, 0, 
			432, 240);

	switch (clevel)
	{
		case  1:
			GpTransBlt(NULL, 
		         &gpDraw[nflip], 
				0, 68, 
				108, 81, 
				(unsigned char*)chooselevel,
				322, 157, 
				432, 240,
				0xff);
			GpTextOut(NULL, &gpDraw[nflip], 120, 190, (char*)"THE BEACH", 0x00);
			current_level = (unsigned char*)level1;

			current_Pal = (GP_PALETTEENTRY*)level1_Pal;
			break;

		case  2:
			GpTransBlt(NULL, 
		         &gpDraw[nflip], 
				107, 68, 
				108, 81, 
				(unsigned char*)chooselevel,
				322, 157, 
				432, 240,
				0xff);
			GpTextOut(NULL, &gpDraw[nflip], 105, 190, (char*)"PARADISE VISION", 0x00);
			current_level = (unsigned char*)level2;
			current_Pal = (GP_PALETTEENTRY*)level2_Pal;
			break;

		case  3:
			GpTransBlt(NULL, 
		         &gpDraw[nflip], 
				213, 68, 
				108, 81, 
				(unsigned char*)chooselevel,
				322, 157, 
				432, 240,
				0xff);
			GpTextOut(NULL, &gpDraw[nflip], 120, 190, (char*)"BLUE NIGHT", 0x00);
			current_level = (unsigned char*)level3;
			current_Pal = (GP_PALETTEENTRY*)level3_Pal;
			break;

		default:
			break;
	}

		GpSurfaceFlip(&gpDraw[nflip++]);
		nflip %= 2;
	}

	GpRectFill(NULL, &gpDraw[nflip], 0, 0, 320, 240, 0x00);
	GpSurfaceFlip(&gpDraw[nflip++]);
	nflip %= 2;

	GpStreamEnd(); // End playing the intro music, it's boring...

	GpPcmPlay((unsigned short*)delete,sizeof(delete), 0);
	GpThreadSleep(50/2); // 50 milliseconds delay

	PlayGame();
}


void PlayGame(void)
{
	int keydata;
	GP_HPALETTE h_pal, old_pal;

	// Create palette
	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)current_Pal);

	// Change palette
	GpPaletteSelect(h_pal,NULL);
	GpPaletteRealize();
		
	// Restore palette
	GpPaletteSelect(old_pal,h_pal);
				
	// Release resource
	GpPaletteDelete(h_pal);
	h_pal = NULL;

	NewGame();

	while (1)
	{
		timer++;

		GpKeyGetEx(&keydata);

		if (keydata & GPC_VK_START)
		{
			GpTextOut(NULL, &gpDraw[nflip^1], 115, 100, (char*)"GAME PAUSED", 0x00);

			// Wait a few milliseconds so that we don't get two START hit
			GpThreadSleep(200/2); // 50 milliseconds delay

			do
			{
				GpKeyGetEx(&keydata);

			} while (!(keydata & GPC_VK_START));

			// Wait a few milliseconds so that we don't get two START hit
			GpThreadSleep(200/2);
		}
		else if (keydata & GPC_VK_SELECT && GpKeyChanged())
		{
			shownext^=1;
		}
		// These needs some explanation: the Gp32 control is so sensitive, we don't
		// want to go right and left when we drop a block (!(keydata & GPC_VK_DOWN))
		// and also if we want precision we need to wait a few milliseconds between
		// each move (timer%x == 0)....
		else if ((keydata & GPC_VK_LEFT) && !(keydata & GPC_VK_DOWN) && (timer%2 == 0))
		{
			block_x--;
			if (!CanPut())
				block_x++;
	     }
		else if (keydata & GPC_VK_RIGHT && !(keydata & GPC_VK_DOWN)  && (timer%2 == 0))
		{
			block_x++;
			if (!CanPut())
				block_x--;
		}
		else if (keydata & GPC_VK_DOWN)
		{
			drop = speed;
		}
		else if (((keydata & GPC_VK_FA) || (keydata & GPC_VK_FR)) && GpKeyChanged())
		{
			RotateBlockRight();
		}
		else if (((keydata & GPC_VK_FB) || (keydata & GPC_VK_FL)) && GpKeyChanged())
		{
			RotateBlockLeft();
		}


		if (timer > speed-drop)
		{
			block_y++;
			if (!CanPut())
			{
				block_y--;
				PutBlock();
				GenerateNew();
			}

			timer = 0;
		}


		if (!CanPut())
		{
			PutBlock();
			GenerateNew();
		}


		DrawMap();
		drop = 0;
	}

}


void GpMain(void *arg)
{
	unsigned int i;
	//char name[12];

	GpSrand(GpTickCountGet());

	for ( i = 0 ; i < 2; i++)
	{
		GpLcdSurfaceGet(&gpDraw[i], i); 
	}
	
	// Set primary surface
	GpSurfaceSet(&gpDraw[0]);	

	// Enable display
	GpLcdEnable();

	// Init SMC access
	GpFatInit();

	GpPcmInit(PCM_M22,PCM_16BIT);
	GpStreamInit(0, 50, 8, 0x400, 4, 0x200);
	
	GpStreamSet("gp:\\gpmm\\gptetris\\bgm.sef",1);	//set a stream data
    	GpStreamPlay();

	//InputName(name);

	// Display the intro screen
	IntroScreen(&gpDraw[0]);

	// Load Hiscores now
	LoadScores();	

	nflip = 1;

	MainMenu();
}
